#include "mccg.h"
#include "spatialg.h"
#include "advisman.h"
#include "ptclgrp.h"
#include "diags.h"

//================= MCCParams Class

MCCParams::MCCParams(SpatialRegionGroup *_SRG)
: ParameterGroup()
{
  name = "MCC";
  SRG = _SRG;

  gas.setNameAndDescription("gas", "name of the gas");
  gas.setValue("NULL");
  parameterList.add(&gas);
  pressure.setNameAndDescription("pressure", "Neutral gas pressure in Torr");
  pressure.setValue("0");
  parameterList.add(&pressure);
  temperature.setNameAndDescription("temperature", "Neutral gas temperature in eV");
  temperature.setValue("0.025");
  parameterList.add(&temperature);
  eSpecies.setNameAndDescription("eSpecies", "electron species for ioniz.");
  eSpecies.setValue("NULL");
  parameterList.add(&eSpecies);
  iSpecies.setNameAndDescription("iSpecies", "ion species for ioniz.");
  iSpecies.setValue("NULL");
  parameterList.add(&iSpecies);
  ionzFraction_i.setNameAndDescription("ionzFraction_i", "num ions created in ionz event");
  ionzFraction_i.setValue("1");
  parameterList.add(&ionzFraction_i);
  ecxFactor.setNameAndDescription("ecxFactor","");
  ecxFactor.setValue("1");
  parameterList.add(&ecxFactor);
  icxFactor.setNameAndDescription("icxFactor","");
  icxFactor.setValue("1");
  parameterList.add(&icxFactor);
};

MCCPackage* MCCParams::CreateCounterPart()
{

	Species* eSpeciesPtr = SRG->get_speciesPtr(eSpecies.getValue());
	Species* iSpeciesPtr = SRG->get_speciesPtr(iSpecies.getValue());
	Species* tSpeciesPtr = SRG->get_speciesPtr(tSpecies.getValue());
	Species* bSpeciesPtr = SRG->get_speciesPtr(bSpecies.getValue());

	MCCPackage* mccPackage = NULL;
	if (gas.getValue() ==(ostring) "Ar") mccPackage = new ArMCC(pressure.getValue(), 
		temperature.getValue(), eSpeciesPtr, iSpeciesPtr, 
		*SRG->advisorp->get_speciesList(), SRG->get_dtBase(), ionzFraction_i.getValue(),
		ecxFactor.getValue(), icxFactor.getValue());
	else if (gas.getValue() ==(ostring) "Ne") mccPackage = new NeMCC(pressure.getValue(), 
		temperature.getValue(), eSpeciesPtr, iSpeciesPtr, 
		*SRG->advisorp->get_speciesList(), SRG->get_dtBase(), ionzFraction_i.getValue(),
		ecxFactor.getValue(), icxFactor.getValue());
	else if (gas.getValue() == (ostring)"Xe") mccPackage = new XeMCC(pressure.getValue(), 
		temperature.getValue(), eSpeciesPtr, iSpeciesPtr, 
		*SRG->advisorp->get_speciesList(), SRG->get_dtBase(), ionzFraction_i.getValue(),
		ecxFactor.getValue(), icxFactor.getValue());
	else if (gas.getValue() == (ostring)"H") mccPackage = new HMCC(pressure.getValue(), 
		temperature.getValue(), eSpeciesPtr, iSpeciesPtr, 
		*SRG->advisorp->get_speciesList(), SRG->get_dtBase(), ionzFraction_i.getValue(),
		ecxFactor.getValue(), icxFactor.getValue());
	else if (gas.getValue() == (ostring)"Li") mccPackage = new LiMCC(pressure.getValue(), 
		temperature.getValue(), eSpeciesPtr, iSpeciesPtr, 
		*SRG->advisorp->get_speciesList(), SRG->get_dtBase(), ionzFraction_i.getValue(),
		ecxFactor.getValue(), icxFactor.getValue());
	else cout << "Unrecognized MCC package requested: " << gas.getValue();
	return mccPackage;
}

void MCCParams::checkRules()
{
	char buffer[512];
   BOOL gasTest = FALSE;
   if (gas.getValue() ==(ostring) "Ar") gasTest = TRUE;
	else if (gas.getValue() ==(ostring) "Ne") gasTest = TRUE;
	else if (gas.getValue() == (ostring)"Xe") gasTest = TRUE;
	else if (gas.getValue() == (ostring)"H") gasTest = TRUE;
	else if (gas.getValue() == (ostring)"Li") gasTest = TRUE;
	if (!gasTest) {
	  sprintf(buffer, "unrecognized gas package: %s", gas.getValue().c_str());
	  ruleMessages.add(new ostring(buffer));
	}
	ostring name = eSpecies.getValue();
	if (name == (ostring)"NULL") ruleMessages.add(new ostring("eSpecies is required!"));
	else if (!SRG->KnownSpecies(name))
	{
		sprintf(buffer, "Unrecognized speciesName: %s", name());
		ruleMessages.add(new ostring(buffer));
	}
	name = iSpecies.getValue();
	if (name == (ostring)"NULL") ruleMessages.add(new ostring("iSpecies is required!"));
	else if (!SRG->KnownSpecies(name))
	{
		sprintf(buffer, "Unrecognized speciesName: %s", name());
		ruleMessages.add(new ostring(buffer));
	}
}


