//spatialg.cpp

#include "advisman.h"
#include "spatialg.h"
#include "abswag.h"
#include "lnchwag.h"
#include "speciesg.h"
#include "maxwelln.h"
#include "fields.h"
#include "diags.h"
#include "mcc.h"
#include "loadg.h"
#include "varloadg.h"
#include "diagg.h"
#include "sptlrgn.h"
#include "conductg.h"
#include "cylaxisg.h"
#include "femitg.h"
#include "bemitg.h"
#include "vbemitg.h"
#include "plsmsrcg.h"
#include "eportg.h"
#include "portteg.h"
#include "porttmg.h"
#include "portgaussg.h"
#include "gapg.h"
#include "eqpotg.h"
#include "foilg.h"
#include "emitprtg.h"
#include "polarg.h"
#include "iloopg.h"
#include "iregiong.h"
#include "varg.h"
#include "spboundg.h"
#include "spemitg.h"
#include "periodic.h"
#include "ptclgrp.h"

// This is not needed, only declare what is needed,
//  WasDumpFileGiven
// #include "xgmini.h"
extern int             WasDumpFileGiven;


void Restore(SpatialRegion*); 

extern "C++" void printf(char *);

#ifdef MPI_VERSION
#include "mpi.h"
extern MPI_Comm XOOPIC_COMM;
extern int MPI_RANK;
#define SRB_ANNOUNCE_TAG 100
#define SRB_LINK_TAG 200
typedef struct {
  char *name;
  int index;
  int linkedP;
} SRBdat;
#endif


//=================== SpatialRegionGroup Class

ostring SpatialRegionGroup::CreateParamGroup(ifstream &fin, ostring &groupname)
{ostring message;
 if (groupname ==(ostring) "Grid")
	 message = GP.InitializeFromStream(fin);
 else if (groupname == (ostring)"Variables")
	 message = VG.InitializeFromStream(fin);
 else if (groupname == (ostring)"Control")
	 message = CP.InitializeFromStream(fin);
 else if (groupname == (ostring)"ParticleGroup")
	 message = PGP.InitializeFromStream(fin);
 else if (groupname == (ostring)"Species")
	 message = readSpecies(fin);
 else if (groupname == (ostring)"MCC")
	 message = AddMCCParamsList(fin);
 else if ( groupname == (ostring)"Load" ) 
		 message = AddLoadParamsList(fin); 
 else if ( groupname == (ostring)"VarWeightLoad" ) 
		 message = AddVarWeightLoadParamsList(fin); 
 else if (groupname == (ostring)"Diagnostic") 
		 message = AddDiagParamsList(fin); 
 else if (advisorp->KnownBoundary(groupname))
	 message = CreateBoundaryParams(fin, groupname);
 else message = "Unknown name ecountered for parameterGroup in input deck";
 return(message);
}

ostring SpatialRegionGroup::CreateBoundaryParams(ifstream &fin, ostring groupname)
{
	BoundaryParams* bptr = constructBoundaryParams(groupname);
	ostring message = bptr->InitializeFromStream(fin);                  
	Boundaries.add(bptr);
	return(message);
}

ostring SpatialRegionGroup::AddMCCParamsList(ifstream& fin)
{
  MCCParams* mPtr = new MCCParams(this);
  ostring message = mPtr->InitializeFromStream(fin);
  MCCParamsList.add(mPtr);
  return message;
}

ostring SpatialRegionGroup::AddLoadParamsList(ifstream &fin)
{
	LoadParams* lptr = new LoadParams(this);
	ostring message = lptr->InitializeFromStream(fin);
	Loads.add(lptr);
	return(message);
}

ostring SpatialRegionGroup::AddVarWeightLoadParamsList(ifstream &fin)
{
	VarWeightLoadParams* lptr = new VarWeightLoadParams(this);
	ostring message = lptr->InitializeFromStream(fin);
	Loads.add(lptr);
	return(message);
}

ostring SpatialRegionGroup::AddDiagParamsList(ifstream &fin)
{
	DiagParams* dptr = new DiagParams(&(this->GP),this); 
	ostring message = dptr->InitializeFromStream(fin);
	Diags.add(dptr);
	return(message);
}

void SpatialRegionGroup::describe()
{
  cout << "testing echoFlag..." << endl;
  if (CP.get_echoFlag()==0) {cout << "echoFlag=0" << endl; return;}
  GP.describe();
  CP.describe();
  PGP.describe();
  {
		oopicListIter<LoadParams> nL(Loads);
		for (nL.restart(); !nL.Done(); nL++)
			nL.current()->describe();
  }
  {
		oopicListIter<DiagParams> nD(Diags);
		for (nD.restart(); !nD.Done(); nD++)
			nD.current()->describe();
  }
  {
		oopicListIter<BoundaryParams> nB(Boundaries);
		for (nB.restart(); !nB.Done(); nB++)
			nB.current()->describe();
  }
}

void SpatialRegionGroup::writeOutputFile(ofstream &fout)
{GP.writeOutputFile(fout);
 CP.writeOutputFile(fout);
 oopicListIter<BoundaryParams> nB(Boundaries);
 for (nB.restart(); !nB.Done(); nB++)
	 {nB.current()->writeOutputFile(fout);}
}

void SpatialRegionGroup::checkRules()
{
}

SpatialRegion* SpatialRegionGroup::CreateCounterPart()
{
	if (CP.getFrandseed())
		frandseed = CP.getFrandseed();
	GP.checkRules();
	GP.showRuleMessages();
	CP.checkRules();
 	CP.showRuleMessages();
	Grid *G;
#ifdef MPI_VERSION
	int MPI_MAX_RANK;
	MPI_Comm_size(XOOPIC_COMM,&MPI_MAX_RANK);
  if (MPI_MAX_RANK > 1) {  //  if we have more than one region.
#ifdef MPI_DEBUG
	 printf("\n%d: Creating grid with %d partitions.",MPI_RANK,MPI_MAX_RANK);
#endif
	 G = GP.CreateCounterPart(CP.getMPIpartition());
  }
  else
#endif /* MPI_VERSION */
	G = GP.CreateCounterPart();

	oopicList<Boundary> *BList = new BoundaryList;
	
	//put the periodic boundary on the boundary list MUST BE FIRST HERE SO THAT IT IS LAST IN PHYSICS
	if ((GP.getPeriodicFlagX1())||(GP.getPeriodicFlagX2())){
		oopicList<LineSegment> *LineSegmentList = new oopicList<LineSegment>;
		LineSegment *newSeg = new LineSegment(Vector2(0,0),Vector2(GP.getJ(),GP.getK()),1);
		LineSegmentList->add(newSeg);
		Boundary* B = 
			new Periodic(LineSegmentList, GP.getPeriodicFlagX1(), GP.getPeriodicFlagX2());
		BList->add(B);
	}
	
	oopicListIter<BoundaryParams> nB(Boundaries);
	for (nB.restart(); !nB.Done(); nB++)
		{
			BoundaryParams *bParam=nB.current();
			bParam->toGrid(G);  //cause the boundary to 'snap to' j,k's if given MKS
			// moved toGrid to checkRules because some of the rules are for j1,j2,k1,k2.
			// moved back to CreateCounter Part and removed checkRules
#ifdef MPI_VERSION
			if(! (bParam->j1.getValue()==-1 && bParam->A1.getValue()==-1)) {
			  // we don't want to create boundaries which are out-of-bounds!
#endif /* MPI_VERSION */
			bParam->checkRules();
			bParam->showRuleMessages();
			Boundary* bPtr = bParam->CreateCounterPart();
			bParam->setBoundaryParams(bPtr);  //should probably cascade CreateCounterPart()
			BList->add(bPtr);
#ifdef MPI_VERSION
			}
#endif /*MPI_VERSION */
		}

	// Move this call here.  It saves trouble with deciding where to put any SRB's.
	G->setBoundaryMask(*BList);
#ifdef MPI_VERSION
	/*  At this point, we should add all the necessary SRB's to our region.  */
	addNeededSRBstoRegion(BList,G);
#endif /*MPI_VERSION*/
	SpeciesParamsList* speciesParamsList = advisorp->get_speciesParamsList();
	ParticleGroupList**	pgListArray = new (ParticleGroupList*[speciesParamsList->nItems()]);
	for (int i=0; i<speciesParamsList->nItems(); i++)
		pgListArray[i] = new ParticleGroupList;

	SpeciesList* speciesList = advisorp->get_speciesList();

	Scalar epsilonR = 1.0;
	Fields *F = new Fields(G, BList, speciesList, epsilonR, 
												 CP.getElectrostaticFlag(),CP.get_presidue(),
												 CP.getBoltzmannFlag(),CP.getSynchRadiationFlag());
	
	F->set_bSpecies(get_speciesPtr(CP.getBoltzSpecies()));
	F->setBNodeStatic(Vector3(CP.getB01(),CP.getB02(),CP.getB03()),
										CP.getbetwig(), CP.getzoff(),CP.getBf().c_str(),
										CP.getB01analytic(),CP.getB02analytic(),CP.getB03analytic());

	
	F->setBeamDump(CP.get_j1BeamDump(), CP.get_j2BeamDump());

	F->setMarderIter(CP.getMarderIter());
	F->setDivergenceCleanFlag(CP.getDivergenceCleanFlag());
	F->setCurrentWeightingFlag(CP.getCurrentWeighting());
	F->set_freezeFields(CP.get_FreezeFields());
	F->setNonRelativisticFlag(CP.getNonRelativisticFlag());
	F->set_nSmoothing(CP.get_nSmoothing());
	F->setInfiniteBFlag(CP.getInfiniteBFlag());
	F->setFieldSubFlag(CP.getFieldSubFlag());
	F->setMarderParameter(CP.getMarderParameter());
	F->setEMdamping(CP.getEMdamping());

	if ((CP.getElectrostaticFlag()) && (CP.getBoltzmannFlag()))
	  {
			Species* species = get_speciesPtr(CP.getBoltzSpecies());
			Boltzmann* boltzmann= new Boltzmann(F,CP.getBoltzmannTemp(), species, CP.getBoltzmannEBFlag());
			F->setBoltzmann(boltzmann);
		 }
	MCC *mcc = 0;
	if (MCCParamsList.nItems() > 0)
	{
	  mcc = new MCC;
	  oopicListIter<MCCParams> mIter(MCCParamsList);
	  for (mIter.restart(); !mIter.Done(); mIter++){
			mIter()->checkRules();
			mIter()->showRuleMessages();
			mcc->addPackage(mIter()->CreateCounterPart());
		}
	}
	
	SpatialRegion *region = new SpatialRegion(F,BList, pgListArray, speciesList, 
															mcc, get_dt());
	region->setParticleLimit(CP.get_particleLimit());
	region->set_np2cFactor(CP.get_np2cFactor());
#ifdef MPI_VERSION
	char buf[512];
	if(Name.getValue()==ostring("Noname") )
	  sprintf(buf,"Region %d",MPI_RANK);
	else
	  sprintf(buf,"%s Region %d",Name.getValue().c_str(),MPI_RANK);
	region->setName(buf);
#else  /* MPI_VERSION */
	region->setName(Name.getValue());
#endif  /* MPI_VERSION */
	oopicList<Diag> *DList = new DiagList;
	oopicListIter<DiagParams> nD(Diags);
	for (nD.restart(); !nD.Done(); nD++){
		nD.current()->toGrid(G);
#ifdef MPI_VERSION
		if(! (nD.current()->j1.getValue()==-1 && nD.current()->A1.getValue()==-1)) {
			  // we don't want to create boundaries which are out-of-bounds!
#endif /* MPI_VERSION */
			Diag* dPtr = nD.current()->CreateCounterPart(region);
			DList->add(dPtr);
#ifdef MPI_VERSION
	}
#endif
	}
	region->setDiagList(DList);

	if (WasDumpFileGiven)	Restore(region);
	//	else 
	ApplyToList(CreateCounterPart(region), Loads, LoadParams);

	F->compute_rho(region->getParticleGroupListArray(),
								 region->get_nSpecies());

	ApplyToList(putCharge_and_Current(region->getTime()),*BList,Boundary);
	/* this line need to follow compute_rho so that the boundary 
		 get rho correct */

	if (CP.getElectrostaticFlag() || !WasDumpFileGiven){
#ifndef MPI_VERSION  /* ES field not working in MPI version yet! */
			F->ElectrostaticSolve(region->getTime(),region->get_dt());
#endif /* MPI_VERSION */
/*		F->toNodes(region->getTime());
		F->setdt(region->get_dt());
		region->particleAdvance(); */  // removed:  reorganized the advance loop.
	}
//	else F->toNodes(region->getTime());
	F->setdt(region->get_dt());

	//  the next two lines set up initial EM fields
	//  which are added to any previous fields.
	//  since it is additive, we should do it only once.
	if(!WasDumpFileGiven) {
	  F->setEinit(CP.getE1init(),CP.getE2init(),CP.getE3init());
	  F->setBinit(CP.getB1init(),CP.getB2init(),CP.getB3init());
	}

	F->toNodes(region->getTime());


// set up the moving window.
        switch(CP.getMovingWindow()){
	  case 1:
	    region->setShift(Vector2(-1,0));
	    break;
	  case 2:
	    region->setShift(Vector2(1,0));
	    break;
	  case 3:
	    region->setShift(Vector2(-1,0));
	    break;
	  case 4:
	    region->setShift(Vector2(1,0));
	    break;
	}
	region->setShiftDelayTime(CP.getShiftDelayTime());

	return region;
}

#ifndef TEXT_OOPIC
#ifndef UNIX
/*
void SpatialRegionGroup::createProbeHistories(GUIConfig* theGUI,
																							SpatialRegion* sr)
{
	{ListIter<DiagnosticParams> nP(Probes);
	 for (nP.restart(); !nP.Done(); nP++)
		 nP.current()->createProbeHistory(theGUI, sr);}
}
*/
#endif
#endif

//-------------------------------------------------------------------
//	temp mechanism to show rule messages - should work on the entire
//	SRG?  Should SRG inherit from ParameterGroup?

void	SpatialRegionGroup::showRuleMessages()
{
	if (GP.ruleMessages.nitems) GP.showRuleMessages();
	if (CP.ruleMessages.nitems) CP.showRuleMessages();
	oopicListIter<BoundaryParams> nB(Boundaries);
	for (nB.restart(); !nB.Done(); nB++)
		{
			BoundaryParams *bp = nB.current();
			if (bp->ruleMessages.nitems) bp->showRuleMessages();
		}
}                               

// the following are used in GeometryEditor::setSpatialRegion
//
GridParams* SpatialRegionGroup::getGridParams()
{
	return &GP;
}

ControlParams* SpatialRegionGroup::getControlParams()
{
	return &CP;
}

oopicList<BoundaryParams>* SpatialRegionGroup::getBoundaryParamsList()
{
	return &Boundaries;
}

oopicList<DiagnosticParams>* SpatialRegionGroup::getProbeParamsList()
{
	return &Probes;
}

// the following are used in GeometryEditor createDragObject
BoundaryParams* SpatialRegionGroup::constructBoundaryParams(ostring groupname)
{
	BoundaryParams* bptr;

	if (groupname == (ostring)"Conductor")
		bptr = new ConductorParams(&GP,this);

	if (groupname == (ostring)"Polarizer")
		bptr = new PolarParams(&GP,this);

	if (groupname == (ostring)"SpatialRegionBoundary")
		bptr = new SpatialRegionBoundaryParams(&GP,this);

	if (groupname == (ostring)"CylindricalAxis")
		bptr = new CylindricalAxisParams(&GP,this);

	if (groupname == (ostring)"FieldEmitter")
		bptr = new FieldEmitterParams(&GP,this);

	if (groupname == (ostring)"FieldEmitter2")
		bptr = new FieldEmitterParams2(&GP,this);

	if (groupname == (ostring)"BeamEmitter")
		bptr = new BeamEmitterParams(&GP,this);

	if (groupname == (ostring)"SingleParticleEmitter")
		bptr = new SingleParticleEmitterParams(&GP,this);

	if (groupname == (ostring)"PlasmaSource")
		bptr = new PlasmaSourceParams(&GP,this);

	if (groupname == (ostring)"EmitPort")
 		bptr = new EmitPortParams(&GP,this);
	
	if (groupname == (ostring)"VarWeightBeamEmitter")
		bptr = new VarWeightBeamEmitterParams(&GP,this);

	if (groupname == (ostring)"ExitPort")
		bptr = new ExitPortParams(&CP, &GP,this);

	if (groupname == (ostring)"Iloop")
		bptr = new IloopParams(&GP, this);

	if (groupname == (ostring)"CurrentRegion")
		bptr = new CurrentRegionParams(&GP,this);

	if (groupname == (ostring)"PortTE")
		bptr = new PortTEParams(&GP,this);

	if (groupname == (ostring)"PortTM")
		bptr = new PortTMParams(&GP,this);

        if (groupname == (ostring)"PortGauss")
                bptr = new PortGaussParams(&GP,this);

	if (groupname == (ostring)"AbsorbWave")
		bptr = new AbsorbWaveParams(&GP,this);

	if (groupname == (ostring)"LaunchWave")
		bptr = new LaunchWaveParams(&GP,this);

	if (groupname == (ostring)"Gap")
		bptr =	new GapParams(&GP,this);

	if (groupname == (ostring)"Equipotential")
		bptr = new EquipotentialParams(&GP,this);
	
	if (groupname == (ostring)"Dielectric")
		bptr = new DielectricParams(&GP,this);

	if (groupname == (ostring)"DielectricRegion")
		bptr = new DielectricRegionParams(&GP,this);

	if (groupname == (ostring)"DielectricTriangle")
		bptr = new DielectricTriangleParams(&GP,this);

	if (groupname == (ostring)"Foil")
		bptr = new FoilParams(&GP,this);

	bptr->setName(groupname);
	return bptr;
}

DiagnosticParams* SpatialRegionGroup::constructProbeParams(ostring groupname)
{
	DiagnosticParams* dptr=0;
#ifndef TEXT_OOPIC
/*
#ifndef UNIX
	if (groupname == (ostring)"E Field")
		dptr = new EFieldProbe(&GP);
	if (groupname == (ostring)"B Field")
		dptr = new BFieldProbe(&GP);
	if (groupname == (ostring)"Current")
		dptr = new CurrentProbe(&GP);
	if (groupname == (ostring)"DivDerror")
		dptr = new DivDerrorProbe(&GP);
	dptr->setName(groupname);
#endif
*/
#endif
	return dptr;
}

ostring 	SpatialRegionGroup::readSpecies(ifstream& fin) 
{
	return advisorp->readSpecies(fin);
}

BOOL SpatialRegionGroup::KnownSpecies(ostring &speciesName)
{
	return advisorp->KnownSpecies(speciesName);
}

Species* SpatialRegionGroup::get_speciesPtr(const ostring &speciesName)
{
	return advisorp->get_speciesPtr(speciesName);
}

SpeciesList* SpatialRegionGroup::get_speciesList()
{
	return advisorp->get_speciesList();
}

SpatialRegionGroup::~SpatialRegionGroup() {
	Boundaries.deleteAll();
	//	Loads.deleteAll();   for moving window, want to keep these around.
	Diags.deleteAll();
	Probes.deleteAll();
	MCCParamsList.deleteAll();
};

#ifdef MPI_VERSION

void SpatialRegionGroup::addNeededSRBstoRegion(oopicList<Boundary> *BList,Grid *G) {
  // Grid has a mask in it so
  // I can figure out where the SRB's need to go!
  int J = G->getJ();
  int K = G->getK();
  int j,k;
  int bcount;  // count of SRB along an edge, needed for naming
  int Js,Ks;   // beginning of the current SRB
  int makingSRB=0;  // flag on whether we're making an SRB or not.
                    //  0->1 transitions set Js, Ks,
                    //  1->0 transitions create the boundary.
  int Nindex;  // index to neighbor array
  int normal;  // normal
  Boundary ***bc1=G->GetBC1();
  Boundary ***bc2=G->GetBC2();
  Boundary *newSRB;

  // if we have a comp. partition below us, try to make SRB
  if(G->neighbor[Nindex=3]!= -1) {  
	 k = 0;
	 makingSRB=0;
	 bcount = 0;
	 normal = 1;
	 for(j=0;j<=J;j++) {
		int BTYPE = FREESPACE;
		if(bc1[j][k]) BTYPE=bc1[j][k]->getBCType();
		if(BTYPE == FREESPACE || BTYPE == DIELECTRIC_BOUNDARY ) {
		  // SRB needed here.
		  if(makingSRB==0) {  //detected an up-transition
			 Js = j; Ks = k; bcount++;
			 makingSRB=1;
		  }
		}
		else if(BTYPE == CONDUCTING_BOUNDARY && makingSRB!=0) {
		  // We've detected the end of a valid SRB:
		  // make it and add it to the BList
		  BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
		  Js=0; Ks=0; makingSRB=0;
		}
		else {
			if(makingSRB) {
				// this is an error codition.
				printf("\n%d:ERROR: SpatialRegionBoundary attempted placement on",MPI_RANK);
				printf("\n%d:ERROR: another SpatialRegionBoundary, a CYLINDRICAL Axis, or",MPI_RANK);
				printf("\n%d:ERROR: a periodic boundary.  NOT ALLOWED!  Exiting.",MPI_RANK);
				printf("\n%d:ERROR: (happened at j=%d, k=%d)  Exiting.",MPI_RANK,j,k);
				abort();
			}
		}
	 }
  }
  // need to back the index up one...
  j--;
  // we've fallen through the loop entirely
  if(makingSRB) {
	 BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
	 makingSRB=0;
  }
 // if we have a comp. partition above us, try to make SRB.
  if(G->neighbor[Nindex=2]!= -1) {  
	 k = K;
	 makingSRB=0;
	 bcount = 0;
	 normal = -1;
	 for(j=0;j<=J;j++) {
		int BTYPE = FREESPACE;
		if(bc1[j][k]) BTYPE=bc1[j][k]->getBCType();
		if(BTYPE == FREESPACE || BTYPE == DIELECTRIC_BOUNDARY ) {
		  // SRB needed here.
		  if(makingSRB==0) {  //detected an up-transition
			 Js = j; Ks = k; bcount++;
			 makingSRB=1;
		  }
		}
		else if(BTYPE == CONDUCTING_BOUNDARY && makingSRB!=0) {
		  // We've detected the end of a valid SRB:
		  // make it and add it to the BList
		  BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
		  Js=0; Ks=0; makingSRB=0;
		}
		else {
			if(makingSRB) {
				// this is an error condition
				printf("\n%d:ERROR: SpatialRegionBoundary attempted placement on",MPI_RANK);
				printf("\n%d:ERROR: another SpatialRegionBoundary, a CYLINDRICAL Axis, or",MPI_RANK);
				printf("\n%d:ERROR: a periodic boundary.  NOT ALLOWED!  Exiting.",MPI_RANK);
				printf("\n%d:ERROR: (happened at j=%d, k=%d)  Exiting.",MPI_RANK,j,k);
				abort();
			}
		}
	 }
  }

  // need to back the index up one...
  j--;
  // we've fallen through the loop entirely
  if(makingSRB) { makingSRB=0;
	 BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
  }
 // if we have a comp. partition left of us, try to make SRB.
  if(G->neighbor[Nindex=1]!= -1) {  
	 j = 0;
	 makingSRB=0;
	 bcount = 0;
	 normal = 1;
	 for(k=0;k<=K;k++) {
		int BTYPE = FREESPACE;
		if(bc2[j][k]) BTYPE=bc2[j][k]->getBCType();
		if(BTYPE == FREESPACE || BTYPE == DIELECTRIC_BOUNDARY ) {
		  // SRB needed here.
		  if(makingSRB==0) {  //detected an up-transition
			 Js = j; Ks = k; bcount++;
			 makingSRB=1;
		  }
		}
		else if(BTYPE == CONDUCTING_BOUNDARY && makingSRB!=0) {
		  // We've detected the end of a valid SRB:
		  // make it and add it to the BList
		  BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
		  Js=0; Ks=0; makingSRB=0;
		}
		else {
			if(makingSRB) {
				// this is an error condition and shouldn't happen.
				printf("\n%d:ERROR: SpatialRegionBoundary attempted placement on",MPI_RANK);
				printf("\n%d:ERROR: another SpatialRegionBoundary, a CYLINDRICAL Axis, or",MPI_RANK);
				printf("\n%d:ERROR: a periodic boundary.  NOT ALLOWED!  Exiting.",MPI_RANK);
				printf("\n%d:ERROR: (happened at j=%d, k=%d)  Exiting.",MPI_RANK,j,k);
				abort();
			}
		}
	 }
  }

  // need to back the index up one...
  k--;
  // we've fallen through the loop entirely
  if(makingSRB) { makingSRB=0;
	 BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
  }
 // if we have a comp. partition right of us, try to make SRB.
  if(G->neighbor[Nindex=0]!= -1) {  
	  j = J;
	  makingSRB=0;
	  bcount = 0;
	  normal = -1;
	  for(k=0;k<=K;k++) {
		  int BTYPE = FREESPACE;
		  if(bc2[j][k]) BTYPE=bc2[j][k]->getBCType();
		  if(BTYPE == FREESPACE || BTYPE == DIELECTRIC_BOUNDARY ) {
			  // SRB needed here.
			  if(makingSRB==0) {  //detected an up-transition
				  Js = j; Ks = k; bcount++;
				  makingSRB=1;
			  }
		  }
		  else if(BTYPE == CONDUCTING_BOUNDARY && makingSRB!=0) {
			  // We've detected the end of a valid SRB:
			  // make it and add it to the BList
			  BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
			  Js=0; Ks=0; makingSRB=0;
		  }
		  else {
			  if(makingSRB) {
				  printf("\n%d:ERROR: SpatialRegionBoundary attempted placement on",MPI_RANK);
				  printf("\n%d:ERROR: another SpatialRegionBoundary, a CYLINDRICAL Axis, or",MPI_RANK);
				  printf("\n%d:ERROR: a periodic boundary.  NOT ALLOWED!  Exiting.",MPI_RANK);
				  printf("\n%d:ERROR: (happened at j=%d, k=%d)  Exiting.",MPI_RANK,j,k);
				  abort();
			  }
		  }
	  }
  }
  // need to back the index up one...
  k--;


  // we've fallen through the loop entirely
  if(makingSRB) {
	 makingSRB=0;
	 BList->add(makeSRB(G,Js,Ks,j,k,normal,G->neighbor,Nindex,bcount));
  }
  

}

Boundary *SpatialRegionGroup::makeSRB(Grid *G,int js,int ks,int je,int ke,int normal,
												  int *neighbor, int Nindex,int bcount) {
  oopicList <LineSegment> *segments = new oopicList<LineSegment>;
  char PartnerName[512];
  SpatialRegionBoundary *newSRB;
  LineSegment *newSeg= new LineSegment(Vector2(js,ks),Vector2(je,ke),normal);
  segments->add(newSeg);
  newSRB= new SpatialRegionBoundary(segments);

  // set up the partner name so boundary matching can happen later.
  // it is in the format "RXXXYYY[V|H]NN"
  //  XXX = higher-rank, YYY = lower-rank, V means vertical, H means horizontal
  //  NN is the section number.  
  sprintf(PartnerName,"R%03d%03d%c%02d",
			 MAX(MPI_RANK,neighbor[Nindex]),
			 MIN(MPI_RANK,neighbor[Nindex]),
			 ((js==je)?'V':'H'),bcount);
  newSRB->PartnerName = PartnerName;
  newSRB->setBoundaryName( PartnerName);
  addtoSpatialRegionBoundaryList(newSRB);
  newSRB->setBoundaryMask(*G);
#ifdef MPI_DEBUG
  printf("\n%d: Creating new SRB: js %d ks %d je %d ke %d normal %d, %s",
			MPI_RANK,js,ks,je,ke,normal,PartnerName);
#endif
return newSRB;
}


  

#endif /* MPI_VERSION */
