//spatialg.h

#ifndef   __PART2G_H
#define   __PART2G_H

#include "param.h"
#include "gridg.h"
#include "controlg.h"
#include "particlg.h"

class DiagnosticParams;
class PlasmaDevice;
class GUIConfig;
class PDistCol;
class PoytFlux;
class Grid;
class Boundary;
class SpatialRegionBoundary;

class AdvisorManager;

#include "loadg.h"
#include "varloadg.h"
#include "boundg.h"
#include "diagg.h"
#include "varg.h"
#include "part1g.h"
#include "mccg.h"

//=================== SpatialRegionGroup Class
// Container class for all ParameterGroups associated with a SpatialRegion

class SpatialRegionGroup: public ParameterGroup
{public:
  GridParams GP;
  ControlParams CP;
  ParticleGroupParams PGP;
  VarGroup VG;
  oopicList<BoundaryParams> Boundaries;
  oopicList<MCCParams> MCCParamsList;
  oopicList<LoadParams> Loads;
  oopicList<DiagParams> Diags;
  oopicList<DiagnosticParams> Probes;
  oopicList<SpatialRegionBoundary> SpatialRegionBoundaryList;
  ostring readSpecies(ifstream& fin); 
  AdvisorManager* advisorp;
  StringParameter Name;
 public:

	SpatialRegionGroup(AdvisorManager* advisor)
	  {  
		 Name.setNameAndDescription("name","A label for the region");
		 Name.setValue("Noname");
		 parameterList.add(&Name);
		 advisorp = advisor;
	  }
	~SpatialRegionGroup();

	ostring CreateParamGroup(ifstream &fin, ostring &groupname);
	// Parameterizes a ParameterGroup object

	ostring CreateBoundaryParams(ifstream &fin, ostring groupname);
	// Parameterizes a BoundaryParams object

	ostring AddMCCParamsList(ifstream& fin);

	ostring AddLoadParamsList(ifstream &fin);
	// Parameterizes a LoadParam object

	ostring AddVarWeightLoadParamsList(ifstream &fin);
	// Parameterizes a VarWeightLoadParam object

	ostring AddDiagParamsList(ifstream &fin);
	// Parameterizes a DiagParam object

	void describe();
	//  Diagnostic for describing object

	void writeOutputFile(ofstream &fout);
	//  Writes output file

	void CreateSpatialRegion();

	void checkRules();

	SpatialRegion*	CreateCounterPart();

	// (JohnV 10-16) Temp mechanisms for handling rule messages
	void showRuleMessages();

	// (JohnV 10-16) Need dt for PlasmaDevice::PlasmaDevice()
	float get_dt() {return CP.getDt();}
  float get_dtBase() {return CP.getDt()*CP.getFieldSubFlag();}

	BoundaryParams* constructBoundaryParams(ostring name);
	DiagnosticParams* constructProbeParams(ostring name);
	GridParams* getGridParams();
	ControlParams* getControlParams();
	PDistCol *createDistributionCollector(BoundaryParams *boundary,Grid *grid,SpeciesList *slist);
	PoytFlux *createDistributionCollector(BoundaryParams *boundary,Grid *grid);
	void SetTimeFunctions(BoundaryParams *boundary,Boundary *B);
	oopicList<BoundaryParams>* getBoundaryParamsList();
	oopicList<DiagnosticParams>* getProbeParamsList();
	Species* get_speciesPtr(const ostring &speciesName);
	BOOL KnownSpecies(ostring &speciesName);
	SpeciesList* get_speciesList();

  void addtoSpatialRegionBoundaryList(SpatialRegionBoundary *srb) {
	 SpatialRegionBoundaryList.add(srb);}

  oopicList<SpatialRegionBoundary> &getSpatialRegionBoundaryList() {
	 return SpatialRegionBoundaryList;}
#ifdef MPI_VERSION
  void addNeededSRBstoRegion(oopicList<Boundary> *BList,Grid *G);
  Boundary *makeSRB(Grid *G,int js,int ks,int je,int ke,int normal,
						  int *neighbor,int Nindex,int bcount);

#endif /* MPI_VERSION */
	 
};                    


#endif  //  __PART2G_H
