/*
====================================================================

diagn.cpp

This file contains all the arrays and functions necessary for maintaining 
and initializing the diagnostics' data.


1.1  (KC 01-6-97) Not calculation the KE of the particles here anymore,
     moved it to the push.  Add KE by species. 
2.01 (Bruhwiler 10-08-99) initialization of synchRadiationFlag added

====================================================================
*/

#include <iostream.h> 
#include <math.h>

#include <stdlib.h>
#include <stdio.h>

#include "fields.h"
#include "grid.h"

#include "ovector.h"
#include <sptlrgn.h>
#include <ptclgrp.h>
#include "globalvars.h"
#include "oopiclist.h"
#include "diagn.h"
#include "gpdist.h"
#include "history.h"
#include "newdiag.h"

#define NPARTICLES 100000

//#define HISTMAX 512    // should really be a power of 2.

#define HISTMAX 4096
//#define HISTMAX 2048    // should really be a power of 2.

//#define MAX(x,y) ( (x) > (y) ? (x) : (y) )


Scalar AllDiagnostics = 1;
Scalar EnerPlots = 1;
Scalar PhaseSpacePlots = 1;

oopicList <Diagnostics> *theDiagnostics;

#ifdef MPI_VERSION
#include "mpi.h"
extern MPI_Comm XOOPIC_COMM;
extern int MPI_RANK;
#define SRB_ANNOUNCE_TAG 100
#define SRB_LINK_TAG 200
#define N_DIAGNOSTIC_TAG 300
typedef struct {
  char *name;
  int index;
  int linkedP;
} SRBdat;
#endif


//  These are the diagnostics arrays:  they're global in scope but used
//  only here and in Maintian_Diagnostics and in xgrafix.

Diagnostics::Diagnostics(SpatialRegion *thisSpace) {
  number_of_species = 1;
  Bz0=0.0;
  divderrormag = 0;
  Show_loaded_densityFlag = FALSE;
  theSpace=thisSpace;
  RegionName = theSpace->getName();
  const int jm = theSpace->getJ();
  const int km = theSpace->getK();
  number_of_species=theSpace->get_nSpecies();
  int j,k,isp;
  B = theSpace->getBNode();
  E = theSpace->getENode();
  blist = theSpace->getBoundaryList();
  BoundaryIhist = new oopicList<Ihistdiag>;
  BoundaryPFhist = new oopicList<PFhistdiag>;
  BDynamic = theSpace->getBNodeDynamic();
  I = theSpace->getI();
  rho = theSpace->getRho();
  boltzmannFlag = theSpace->getBoltzmannFlag();
  electrostaticFlag = theSpace->getElectrostaticFlag();
  synchRadiationFlag = theSpace->getSynchRadiationFlag();
  if (boltzmannFlag){
		totalRho = theSpace->getTotalRho();
	}
  loaded_density = theSpace->getloaddensity();
  Show_loaded_densityFlag = theSpace->getShowInitialDensityFlag();
  phi = theSpace->getPhi();
  divderror = theSpace->getDivDerror();
  rho_species = theSpace->get_rho_species();
  theSpecies= new SpeciesDiag[number_of_species];
  CellVolumes=theSpace->get_halfCellVolumes();

  // get memory for the grid arrays
  x1_array = new Scalar[jm+2];
  x2_array = new Scalar[km+2];
  //get memory for the diagnostics

  S_array= new Vector3 *[jm+2];
  Ue= new Scalar * [jm+2];
  Ub= new Scalar * [jm+2];
		
#ifdef DEBUG_PHI
		phi_err = new Scalar *[jm+2];
#endif

  // Set up the user-defined diagnostics
  oopicListIter<Diag> DiagIter(*thisSpace->getDiagList());
  for(DiagIter.restart();!DiagIter.Done();DiagIter++) 
	 DiagIter.current()->setDiagnostics(this);
  
  for(j=0;j<=jm;j++)
	 {
		S_array[j]= new Vector3 [km+2];
		for(k=0;k<=km;k++)
			S_array[j][k] = Vector3(0,0,0);

		Ue[j]=new Scalar[km+2];
		memset(Ue[j],0,(km+2)*sizeof(Scalar));

		Ub[j]=new Scalar[km+2];
		memset(Ub[j],0,(km+2)*sizeof(Scalar));
		
#ifdef DEBUG_PHI
		phi_err[j] = new Scalar[km+2];
#endif

	 }

  //  Get memory for the arrays that contain particle positions for xgrafix

  for(isp=0;isp<number_of_species;isp++) {
	 // search for the name corresponding to this species
	 oopicListIter<Species> siter(*theSpace->getSpeciesList());
	 for(siter.restart();!siter.Done();siter++) {
		if(isp==siter.current()->getID()) {
		  theSpecies[isp].name=siter.current()->get_name_ptr();
		  theSpecies[isp].spec = siter.current();
		  theSpecies[isp].ID = siter.current()->getID(); 
		}
	 }

	 theSpecies[isp].memory_size=NPARTICLES;
	 theSpecies[isp].nparticles=0;
	 theSpecies[isp].nparticles_plot = 0;
	 theSpecies[isp].x1positions= new Scalar[NPARTICLES];
	 theSpecies[isp].x2positions= new Scalar[NPARTICLES];
	 theSpecies[isp].x1velocities=new Scalar[NPARTICLES];
	 theSpecies[isp].x2velocities=new Scalar[NPARTICLES];
	 theSpecies[isp].x3velocities=new Scalar[NPARTICLES];
	 memset(theSpecies[isp].x1positions,0,NPARTICLES*sizeof(Scalar));
	 memset(theSpecies[isp].x2positions,0,NPARTICLES*sizeof(Scalar));
	 memset(theSpecies[isp].x1velocities,0,NPARTICLES*sizeof(Scalar));
	 memset(theSpecies[isp].x2velocities,0,NPARTICLES*sizeof(Scalar));
	 memset(theSpecies[isp].x3velocities,0,NPARTICLES*sizeof(Scalar));
  }


  number = new Scalar_History* [number_of_species];
	ke_species = new Scalar_History* [number_of_species];
  ngroups = new Scalar_History* [number_of_species];
  total_density = new Scalar_History* [number_of_species];
  Ave_KE = new Scalar_History* [number_of_species];
  for(isp=0;isp<number_of_species;isp++) {
		number[isp]= new Scalar_History(HISTMAX,2);
		ke_species[isp] = new Scalar_History(HISTMAX,2);
		ngroups[isp] = new Scalar_History(HISTMAX,2);
		total_density[isp] = new Scalar_History(HISTMAX,2); 
		Ave_KE[isp] = new Scalar_History(HISTMAX,2);
	}

  //  Set up the grid arrays

  for(j=0;j<=jm;j++)
	 x1_array[j]=theSpace->getMKS(j,0).e1();

  for(k=0;k<=km;k++)
	 x2_array[k]=theSpace->getMKS(0,k).e2();

  //  get the size of the grid
  x1max = theSpace->getMKS(jm,km).e1();
  x2max = theSpace->getMKS(jm,km).e2();
  x1min = theSpace->getMKS(0,0).e1();
  x2min = theSpace->getMKS(0,0).e2();

  //set up diagnsotic history arrays for boundaries which
  //have particle diagnostics on
  oopicListIter<Boundary> nextb(*blist);
  for(nextb.restart();!nextb.Done();nextb++)
	 { //if it's taking diagnostics of particles
		if(nextb.current()->get_particle_diag())
		  {
			 Ihistdiag *Ihistory= new Ihistdiag;
			 Ihistory->Ihist = new Scalar_History(HISTMAX,2);
			 Ihistory->p_dist = nextb.current()->get_particle_diag();
			 strcpy(Ihistory->name,nextb.current()->getBoundaryName().c_str());
			 BoundaryIhist->add(Ihistory);
		  }
	 }
  for(nextb.restart();!nextb.Done();nextb++)
	 { 
		if(nextb.current()->getPF())
		  {
			 PFhistdiag *PFhistory= new PFhistdiag;
			 PFhistory->PFhist = new Scalar_History(HISTMAX,2);
			 PFhistory->PFlocal = new Scalar_Local_History(HISTMAX,MAX(1,HISTMAX/16));
			 PFhistory->p_flux = nextb.current()->getPF();
			 strcpy(PFhistory->name,nextb.current()->getBoundaryName().c_str());
			 BoundaryPFhist->add(PFhistory);
		  }
	 }
}


/************************************************************************/
/* time history accumulator; calculates and stores all history values,  */
/* and performs combing on history values when low on memory            */

void Diagnostics::history()
{
	register int isp, k, j;

	/*****************************************/
	/*  Fixing the diagnostic arrays         */

	simulation_time=theSpace->getTime();
	int jm=theSpace->getJ();
	int km=theSpace->getK();
	oopicListIter<Ihistdiag> nextdiag(*BoundaryIhist);
	oopicListIter<PFhistdiag> nextdiag2(*BoundaryPFhist);

	DiagList *dlist = theSpace->getDiagList();
	oopicListIter<Diag> nextd(*dlist);

	// ------Updating newDiag-----//
	for(nextd.restart();!nextd.Done(); nextd++) 
		nextd.current()->MaintainDiag((Scalar)simulation_time);
	//--------------------------//

	// update the history objects

	Uktot = 0;
	for(isp=0;isp<number_of_species;isp++) {
		number[isp]->add(theSpecies[isp].nparticles,simulation_time);
		ngroups[isp]->add(theSpecies[isp].ngroups,simulation_time);
		ke_species[isp]->add(theSpecies[isp].spec->getKineticEnergy(),simulation_time);
		Uktot += theSpecies[isp].spec->getKineticEnergy();
	}
	energy_e.add(Uetot,simulation_time);
	energy_b.add(Ubtot,simulation_time);
	energy_k.add(Uktot,simulation_time);
	energy_all.add(Uetot+Ubtot+Uktot,simulation_time);
	divderrorhis.add(divderrormag,simulation_time);

	for(nextdiag2.restart();!nextdiag2.Done();nextdiag2++)
		{ //if it's taking diagnostics of Poyting Flux
			Scalar p_flux = *nextdiag2.current()->p_flux;
			nextdiag2.current()->PFhist->add(p_flux,simulation_time);
			nextdiag2.current()->PFlocal->add(p_flux,simulation_time);
		}
	for(nextdiag.restart();!nextdiag.Done();nextdiag++)
		{ //if it's taking diagnostics of particles
			Scalar Ir = nextdiag.current()->p_dist->getdQ()/dt;
			nextdiag.current()->Ihist->add(Ir,simulation_time);
		}


	Scalar total_dens = 0.0;
	Scalar icharge = 0.0;
	for(isp=0;isp<number_of_species;isp++) {
		total_dens = 0.0;
		icharge = 1/theSpecies[isp].spec->get_q(); 
		for (j=0; j<=jm; j++)
			for (k=0; k<=km; k++)
				total_dens += CellVolumes[j][k]*rho_species[isp][j][k];

		total_dens *= icharge;

		total_density[isp]->add(total_dens,simulation_time);
		if (total_dens)
			Ave_KE[isp]->add(theSpecies[isp].spec->getKineticEnergy()/(fabs(ELECTRON_CHARGE)*total_dens), simulation_time);
		else
			Ave_KE[isp]->add(0.,simulation_time);
	}
}

/* another function for testing. */
void Diagnostics::compute_div_derror_magnitude(Scalar **divderror,int jm,int km) {
	int j,k;
	Scalar rhomagave;
	divderrormag=0;
	rhomagave = 0;
	for(j=1;j<jm-1;j++)
		for(k=1;k<km-1;k++) {
			rhomagave+=fabs(rho[j][k]);
			divderrormag+=fabs(divderror[j][k]);
		}
	if(rhomagave!=0) divderrormag/=rhomagave;
}


/*  UpdatePreDiagnostics does processing on raw physics data such
	which should be done before a physics iteration so that the user
	gets fields + particle positions which are reasonably synchronized.
 */

void Diagnostics::UpdatePreDiagnostics()
{  
	int i,j;

#ifndef PHI_TEST

	if(AllDiagnostics) {  //collect diagnostics at all?
		 
		if(PhaseSpacePlots!=0.0) {
			int isp;
			int skip;
			//  maintain the phase space diagnostics.  
			for(isp = 0; isp < number_of_species; isp++) {
				SpeciesDiag *thisSpc= &theSpecies[isp];
				thisSpc->nparticles=0;
				int pgnumparticles;
				oopicListIter<ParticleGroup> 
					pgscan(*theSpace->getParticleGroupListArray()[isp]);
				i=0;  //place in phase space diagnostic


				// First count the particles, store in thisSpc=theSpecies[isp]
				for(thisSpc->ngroups=0,pgscan.restart();!pgscan.Done();pgscan++,thisSpc->ngroups++){
					pgnumparticles=(int)pgscan.current()->get_n();
					thisSpc->nparticles+=pgnumparticles;
				}

				// Skip particles in the physics so we don't overflow the
				// particle array in the diagnostics
				skip = thisSpc->nparticles/thisSpc->memory_size +1;
				thisSpc->nparticles_plot = thisSpc->nparticles/skip;
#ifdef MPI_VERSION
				Scalar l_nparticles = thisSpc->nparticles;
				Scalar t_nparticles = 0;
				MPI_Reduce ( &l_nparticles, &t_nparticles,1,MPI_Scalar,MPI_SUM,0,XOOPIC_COMM);
				if(MPI_RANK==0)
				  thisSpc->nparticles = (int)t_nparticles;
#endif /* MPI_VERSION */
				
				for(pgscan.restart();!pgscan.Done();pgscan++) {
					int npart =(int) pgscan.current()->get_n();
					for(j=0;j<npart&&i<NPARTICLES;j+=skip,i++) {
							Vector2 *xarr=pgscan.current()->get_x();
							Vector3 *varr=pgscan.current()->get_u();
							Vector2 x = theSpace->getMKS(xarr[j]);
							Vector3 v = varr[j];
							
							thisSpc->x1positions[i]=x.e1();
							thisSpc->x2positions[i]=x.e2();
							thisSpc->x1velocities[i] = v.e1();
							thisSpc->x2velocities[i] = v.e2();
							thisSpc->x3velocities[i] = v.e3();
						}
				}
			}
		}
		
	}
#endif
}

void Diagnostics::UpdatePostDiagnostics() {
  int j,k;
  int jm = theSpace->getJ();
  int km = theSpace->getK();

	if (boltzmannFlag)
		theSpace->CopyBoltzmannRho();

  //  initialize time history varaibles to zero
  Uetot=0; Ubtot=0;
  if(AllDiagnostics) {  //collect diagnostics at all?
		Grid* grid = theSpace->get_grid();
		theSpace->updateDivDerror();

		if(EnerPlots){
			for( j=0;j<=jm;j++)
				for( k=0;k<=km;k++){

					/* get the E energy density */
					Ue[j][k] = 0.5*(theSpace->get_fields()->get_epsi(MIN(j,jm-1),MIN(k,km-1))+
													theSpace->get_fields()->get_epsi(MAX(j-1,0),MIN(k,km-1))+
													theSpace->get_fields()->get_epsi(MIN(j,jm-1),MAX(k-1,0))+
													theSpace->get_fields()->get_epsi(MAX(j-1,0),MAX(k-1,0)))/4.0
														*(E[j][k]*E[j][k]);
					/* get the B energy density */
					Ub[j][k]=sqr(BDynamic[j][k].e1());
					Ub[j][k]+=sqr(BDynamic[j][k].e2());
					Ub[j][k]+=sqr(BDynamic[j][k].e3());
					Ub[j][k]*=0.5*iMU0;

#ifdef DEBUG_PHI
					if(phi[j][k] > 1e-20)
						//absolute error (uncomment one)
					phi_err[j][k] = fabs((phi[j][k] - (sin( M_PI * grid->getX()[j][k].e1()) 
						* sin(M_PI*grid->getX()[j][k].e2()))));

						//relative error
//					phi_err[j][k] = fabs((phi[j][k] - (sin( M_PI * grid->getX()[j][k].e1()) 
//						* sin(M_PI*grid->getX()[j][k].e2())))/phi[j][k]);
					else phi_err[j][k] = 0;
#endif
					
					//collect totals of energy for volumetric data
					Uetot+=Ue[j][k]*CellVolumes[j][k];
					Ubtot+=Ub[j][k]*CellVolumes[j][k];
				}
			
			// This section makes the poyting vector on the interior points. 
			// To get the edge points it is a little harder.
			if (!electrostaticFlag){
				for( j=0;j<=(jm-1);j++)
					for( k=1;k<=(km-1);k++)
						S_array[j][k] = ((E[j][k].cross(BDynamic[j][k]))).
							jvMult(grid->dS(j,k))*iMU0;
			}
		}

		history();
		
	}
} 
#ifdef BENCHMARK
//  This function writes a 'trace', 
//  the first component of the electric field
//  so that you can check the output of a simulation
//  which has no GUI.

void write_validation() {
  FILE *trace_file;
  int J = theSpace->getJ();
  int K = theSpace->getK();
  if((trace_file=fopen("trace.dat","w"))==NULL) exit(1);
  for(int j=0;j<J;j++) 
    for(int k=0;k<K;k++) 
			fprintf(trace_file,"%10.4g\n",E[j][k].e1());

  fclose(trace_file);
}
#endif




