#include "fft.h"

#ifndef _DIAGN_H
#define _DIAGN_H

#ifndef UNIX
#include <precision.h>
#include "ovector.h"
#endif

/*
====================================================================

DIAGN.H

Histories.

Revision/Author/Date
2.01  (Bruhwiler, 10-08-99) added integer flag synchRadiationFlag.
====================================================================
*/

#include "history.h"
#include "ostring.h"

class Scalar_Local_History;
class Scalar_History;

class PDistCol;
class Species;

typedef struct {
	const char *name;
	long unsigned int memory_size;
	int nparticles;
	int ngroups;  //number of particle groups
	int nparticles_plot;  // The number of particles actually plotted.
	int ID;   //  position in the particlegrouplist.  
	Scalar *x1positions;
	Scalar *x2positions;
	Scalar *x1velocities;
	Scalar *x2velocities;
	Scalar *x3velocities;
	Species *spec;
} SpeciesDiag;

typedef struct {
	char name[80];
	PDistCol *p_dist;
	Scalar_History *Ihist;
} Ihistdiag;


typedef struct {
	char name[80];
	Scalar *p_flux;
	Scalar_History *PFhist;
	Scalar_Local_History *PFlocal;
} PFhistdiag;

	
class Diagnostics {
public:

  //  for the initwin function
ostring X1;
  ostring X2;
  ostring X3;
  ostring E1;
  ostring E2;
  ostring E3;
  ostring B1;
  ostring B2;
  ostring B3;
  ostring I1;
  ostring I2;
  ostring I3;
  ostring U1vsX1;
  ostring U2vsX1;
  ostring U3vsX1;
  ostring U1vsX2;
  ostring U2vsX2;
  ostring U3vsX2;

  int jm,km,km2;
  SpeciesDiag* theSpecies;
  Scalar Uetot, Ubtot, Uktot, divderrormag;
  Scalar *x1_array, *x2_array, **E1_array, **E2_array,**E3_array;
  Scalar **B1_array, **B2_array, **B3_array, **I1_array, **I2_array,**I3_array;
  Vector3 **B,**E,**S,**I,**BDynamic;
  Vector3 **S_array;
  Scalar **rho,**phi, **totalRho;
#ifdef DEBUG_PHI
  Scalar **phi_err;
#endif
  Scalar ***rho_species, ***loaded_density;
  Scalar **CellVolumes;
  int Show_loaded_densityFlag;
  Scalar x1max,x2max,x1min,x2min;
  SpatialRegion *theSpace;
  Scalar **Ue, **Ub;
  Scalar **divderror;
  Scalar *I1hist;
  Scalar **E1_PHI,**E2_PHI,**E1_ERR,**E2_ERR;
  Scalar **ave_den_at_Z,**ave_den_at_Z_show;

  ostring RegionName;

  Scalar_History energy_e, energy_b, energy_k, energy_all, **number, **ke_species,
	 **ngroups, **total_density, **Ave_KE;
  Scalar_History divderrorhis;
  Scalar *t_array;

  Scalar **error;
  int number_of_species;
  int electrostaticFlag;
  int synchRadiationFlag;
  int boltzmannFlag;
  Scalar Bz0;
  BoundaryList *blist;
  oopicList<Ihistdiag> *BoundaryIhist;
  oopicList<PFhistdiag> *BoundaryPFhist;
  Diagnostics(SpatialRegion *thisSpace);
  void UpdatePreDiagnostics();
  void UpdatePostDiagnostics();
  void DumpDiagnostics();
  void RestoreDiagnostics(); 
  void Update() {
	 UpdatePreDiagnostics();
	 UpdatePostDiagnostics();
  }
  void history();
  ~Diagnostics(){};
  void InitWin();
  void set_analytic_rho(Scalar**, int, int, Scalar, Scalar, int);
  void compute_div_derror_magnitude(Scalar **divderror, int jm, int km);
};

extern oopicList<Diagnostics> *theDiagnostics;

#endif //DIAGN_H
