#include "sptlrgn.h"
#include "globalvars.h"
#include <stdio.h>

// #include "xgmini.h"
extern char            theDumpFile[80];

//#include "diagn.h"
//#include "newdiag.h"
//#include "ptclgrp.h"
#include <iostream.h>
#include "plsmadev.h"
#include "dump.h"

extern PlasmaDevice *theDevice;
extern oopicList<SpatialRegion> *theRegions;
extern "C" {
  void Dump(char *);
  void Quit(void);
}


#ifdef MPI_VERSION
#include "mpi.h"
extern int MPI_RANK;
#endif
 

/***************************************************************/
/* Dumping the current state of the system into a binary file. */
/* Note: the binary file is written in the format used by IBM  */
/* and DEC compatible machines (low bytes followed by high     */
/* bytes). For this Dump() and Restore() call the functions    */
/* sun_read() and sun_write().                                 */

char Revision[]={'2','.','6','0'};
// Doing a re-org of dump.
// calling plasma device and cascading from there.

void Dump(char *the_dump_file_name)
{

	FILE *DMPFile;
	char theDumpFileName[256];
	
#ifdef MPI_VERSION
	sprintf(theDumpFileName,"%s.%d",the_dump_file_name,MPI_RANK);
#else
  sprintf(theDumpFileName,"%s",the_dump_file_name);
#endif
  if ((DMPFile = fopen(theDumpFileName, "w")) == NULL)
		{
			printf("filename = %s\n", theDumpFile);
			puts("Dump: open failed error#1");
			return;
		}
	
	XGWrite(Revision, 1, 4, DMPFile, "char");
	
	// Cascade from Plasma Device
	
	theDevice->Dump(DMPFile);
	
	fclose(DMPFile);
	


}
/****************************************************************/

void Restore(SpatialRegion *region)
{

  char Rev[5];
  char FileName[256];
  FILE *DMPFile=NULL;
  int i=0;
  
  // All of the files of the form
  // theDumpFile
  // theDumpFile.1
  // theDumpFile.2
  // ..
  // theDumpFile.n
  // have to be checked.

  DMPFile = fopen(theDumpFile, "r+b");
  do {
	  if(DMPFile==NULL && i==0)  {
		  sprintf(FileName,"%s.%d",theDumpFile,i);
		  DMPFile = fopen(FileName,"r+b");
		  if(DMPFile==NULL) {
			  printf("Can't read dump file.");
			  return;
		  }
			  
	  }
	  if(i>0) {
		  sprintf(FileName,"%s.%d",theDumpFile,i);
		  DMPFile = fopen(FileName,"r+b");
		  if(DMPFile == NULL) return;
	  }		  
		  
	  
	  
	  XGRead(Rev, 1, 4, DMPFile, "char");

	  // restore version 2.00 dump files. 
	  if(strncmp(Rev,"2.00",4)==0) {
		  region->Restore_2_00(DMPFile);
		  return;
	  }

	  // check to see if it's a current verson
	  if(strncmp(Rev,"2.60",4)!=0) {
		  printf("Dump:  unknown version.  Exiting.");
		  exit(1);
	  }

	  cout << "Started reading dump file." << endl;
	  region->Restore(DMPFile);
	  cout << "Finished reading dump file." << endl;

	  fclose(DMPFile);
	  i++;

  } while(DMPFile != NULL);  // may as well be a while(1)--exits before if DMPFile==NULL

}


/* restore the previous verson of the dump file. */
void Restore_2_00(SpatialRegion *region)
{

  char Rev[5];
  static FILE *DMPFile=NULL;
	int i;

  if(DMPFile==NULL) {
	 if ((DMPFile = fopen(theDumpFile, "r+b")) == NULL)
		{
			 puts("Dump: open failed error #2");
			 printf("filename %s", theDumpFile);
			 return;
		 }
  }

  XGRead(Rev, 1, 4, DMPFile, "char");
  for (i=0; i<4; i++)
		if (Rev[i]!=Revision[i]){
			puts("Incompatible dump file version");
			putchar(7);
			exit(1);
		}

	cout << "started reading dump file." << endl;
	region->Restore_2_00(DMPFile);
	cout << "finished reading dump file." << endl;

	fclose(DMPFile);


}


/****************************************************************/

void Quit(void)
{
 delete theDevice;
#ifdef MPI_VERSION
 MPI_Finalize();
#endif
}

void XGVector2Write(Vector2& vector, FILE *DMPFile)
{
	Scalar temp;
	
	for (int e=1; e<=2;e++){
		temp = vector.e(e);
		XGWrite(&temp, ScalarInt, 1, DMPFile, ScalarChar);
	}
}

void XGVector3Write(Vector3& vector, FILE *DMPFile)
{
	Scalar temp;

	for (int e=1; e<=3;e++){
		temp = vector.e(e);
		XGWrite(&temp, ScalarInt, 1, DMPFile, ScalarChar);
	}
}

void XGVector2Read(Vector2& vector, FILE *DMPFile)
{
	Scalar temp;
	
	for (int e=1; e<=2;e++){
		XGRead(&temp, ScalarInt, 1, DMPFile, ScalarChar);
		vector.set(e,temp);
	}
}

void XGVector3Read(Vector3& vector, FILE *DMPFile)
{
	Scalar temp;
	
	for (int e=1; e<=3;e++){
		XGRead(&temp, ScalarInt, 1, DMPFile, ScalarChar);
		vector.set(e,temp);
	}
}
